--[[
	Tools for managing calculation elements in NavComp
]]

function navcomp:SetPath (path, clearPath)
	local v
	if path and #path > 0 then
		if clearPath == nil then clearPath = true end
		if GetCurrentSectorid () == path [1] then
			table.remove (path, 1)
		end
		if clearPath then NavRoute.clear () end
		for _,v in ipairs (path) do
			NavRoute.addbyid (v)
			navcomp:Yield ()
		end
	end
end

function navcomp:ComparePaths (path1, path2)
	if type (path1) ~= "table" or type (path2) ~= "table" then return false
	elseif #path1 ~= #path2 then return false
	else
		-- Compare sectorIds in each path.  If any differ, return false
		local k
		for k=1, #path1 do
			if path1 [k] ~= path2 [k] then return false end
		end
		
		return true
	end
end

function navcomp:ClearActivePath ()
	navcomp.data.activePath.name = nil
	navcomp.data.activePath.note = ""
	navcomp.data.activePath.autoPlot = false
	navcomp.data.activePath.autoReload = false
end

function navcomp:CheckAutoPlot (navpath)
	if navpath.autoPlot or navcomp.data.autoPlot then
		navcomp:PlotPath ()
	end
end

function navcomp:SaveActivePath (data)
	local navpath = navcomp.data:CreatePath (data)
	if navpath then
		navcomp.data:SavePathNote (navpath)
		navcomp.data:SavePathNotes ()
		navcomp:CheckAutoPlot (navpath)
		return true
	else
		return false
	end
end

function navcomp:LoadActivePath (pathName)
	local navpath = navcomp.data:LoadPathNote (pathName) or navcomp.data.activePath
	if navpath and #navpath.path > 0 then
		navcomp.data.activePath = navpath
		navcomp:SetPath (navpath.path)
		navcomp:CheckAutoPlot (navpath)
		return true
	else
		return false
	end
end

function navcomp:BuildLocationId (x, y)
	return 16 * (x - 1) + y
end

function navcomp:BuildSectorId (sysId, x, y)
	return 256 * (sysId-1) + navcomp:BuildLocationId (x, y)
end

function navcomp:Distance (s1, s2)
	local _, y1, x1 = SplitSectorID (s1)
	local _, y2, x2 = SplitSectorID (s2)
	return (x2 - x1)*(x2 - x1) + (y2 - y1)*(y2 - y1)
end

function navcomp:Lower (n, r)
	if n < r + 1 then
		return n - 1
	else
		return r
	end
end

function navcomp:Upper (n, r)
	if n > 16 - r then
		return 16 - n
	else
		return r
	end
end

function navcomp:GetSectorNote (s1)
	local sysId = GetSystemID (s1)
	return SystemNotes [sysId][s1] or ""
end

function navcomp:SyncSectorNotes ()
	local sysId, targetId, sectorId, notes, note
	local avoids = navcomp.data:Clone (navcomp.data.avoidSectors)
	navcomp.data.avoidSectors = {}
	navcomp.data.anchors = {}
	
	-- Update the anchor sector list
	for sysId=1, #SystemNames, 1 do
		notes = SystemNotes [sysId]
		for targetId, note in pairs (notes) do
			-- Check for manual avoids
			if string.find (note:lower (), "#avoid:") then
				if not avoids [targetId] then
					-- new avoid, clear system from cache
					navcomp.data.stressMaps [GetSystemID (targetId)] = nil
				end
				navcomp.data.avoidSectors [targetId] = "YES"
			end
			-- Check for Anchors
			-- Anchor syntax
			-- #anchors (<sectorId>, <sectorId>, ...):
			local sectors = string.match (note, "#anchors%s*%(([%*?%a%-?%d%d?%s*,?]+)%):")
			if sectors then
				local v, h, sectorId, override
				navcomp.data.anchors [targetId] = {}
				for override, v, h in string.gmatch (sectors, "(%*?)(%a)%-?(%d%d?)") do
					override = override == "*"
					sectorId = navcomp:BuildSectorId (sysId, h, navcomp.data.sectors [v:upper ()])
					table.insert (navcomp.data.anchors [targetId], {s=sectorId, override=override})
				end
			end
		end
	end
	
	-- Remove all changed systems from stress map due to changes in avoid sectors
	for sectorId, _ in pairs (avoids) do
		if not navcomp.data.avoidSectors [sectorId] then
			-- Removed old avoid, clear system from cache
			navcomp.data.stressMaps [GetSystemID (sectorId)] = nil
		end
	end
end

function navcomp:GetSectorLine (s1, s2)
	local s
	local sysId, y1, x1 = SplitSectorID (s1)
	local _, y2, x2 = SplitSectorID (s2)
	local sysTag = 256 * (sysId - 1)
	local loc1 = navcomp:BuildLocationId (x1, y1)
	local loc2 = navcomp:BuildLocationId (x2, y2)
	local lineId
	if loc1 <= loc2 then
		lineId = string.format ("%d:%d", loc1, loc2)
	else
		lineId = string.format ("%d:%d", loc2, loc1)
	end
	local sectors = {}
	
	if navcomp.data.sectorLines [lineId] then
		for _,s in ipairs (navcomp.data.sectorLines [lineId]) do
			--sectors [#sectors + 1] = sysTag + s
			table.insert (sectors, sysTag + s)
		end
	else
		navcomp:Yield ()
		local diffx = x2 - x1
		local diffy = y2 - y1
		local ang = math.atan2 (diffx, diffy)
		local dist = math.sqrt (diffx * diffx + diffy * diffy)
		
		local newX, newY
		local sine = math.sin (ang)
		local cosine = math.cos (ang)
		local lineStr, sectorId, k
		for k=0, math.floor (dist + 0.5), 0.25 do
			newX = math.floor (k * sine + x1 + 0.5)
			newY = math.floor (k * cosine + y1 + 0.5)
			sectorId = navcomp:BuildSectorId (sysId, newX, newY)
			lineStr = "|" .. table.concat (sectors, "|") .. "|"
			if not string.find (lineStr, "|" .. sectorId .. "|") then
				table.insert (sectors, sectorId)
			end
			navcomp:Yield ()
		end
		
		navcomp.data.sectorLines [lineId] = {}
		for _,s in ipairs (sectors) do
			table.insert (navcomp.data.sectorLines [lineId], (s - sysTag))
		end
		navcomp.data.isLineDataSaved = false
	end
	
	return sectors
end

function navcomp:CheckBotTypes (s1, types)
	local bots
	if type (s1) == "number" then
		bots = GetBotSightedInfoForSector (s1):lower ()
	else
		bots = s1:lower ()
	end
	local v
	for _,v in ipairs (types) do
		if string.find (bots, v) then return true end
	end
	
	return false
end

function navcomp:IsEncounteredStormSector (s1)
	if string.find (navcomp.data.stormSectorStr, "|" .. tostring (s1) .. "|") then
		return true
	else
		return false
	end
end

-- Check record for hostile bots
function navcomp:IsEncounteredBotSector (s1)
	if string.find (navcomp.data.botSectorStr, "|" .. tostring (s1) .. "|") then
		return true
	else
		return false
	end
end

function navcomp:IsAvoidSector (s1)
	return (navcomp.data.avoidSectors [s1] ~= nil)
end

-- Hostiles listed in nav screen
function navcomp:IsHostileBotSector (s1)
	return navcomp:CheckBotTypes (s1, navcomp.data.botAvoidTypes)
end

-- Benign bots only.  Check listed and recorded
function navcomp:IsBotSector (s1)
	return navcomp:CheckBotTypes (s1, navcomp.data.botSafeTypes) and 
				not navcomp:CheckBotTypes (s1, navcomp.data.botAvoidTypes) and 
				not navcomp:IsEncounteredBotSector (s1)
end

-- Special Stations
function navcomp:IsStationSector (s1)
	return navcomp:IsConquerableStationSector (s1) or navcomp:CheckBotTypes (s1, navcomp.data.botStationTypes)
end

function navcomp:IsConquerableStationSector (s1)
	if navcomp.data.conquerableStations [s1] then return true
	else return false
	end
end

function navcomp:IsRacetrackSector (s1)
	if navcomp.data.racetracks [s1] then return true
	else return false
	end
end

-- Special Sectors
function navcomp:IsStarSector (s1)
	local sysName = SystemNames [GetSystemID (s1)]:lower ()
	return string.find (navcomp.data.starSectors [sysName], "|" .. tostring (s1) .. "|")
end

function navcomp:IsTrainingSector (s1)
	return string.find (navcomp.data.training, "|" .. tostring (s1) .. "|")
end

function navcomp:IsWormholeSector (s1)
	return string.find (navcomp.data.whSectorStr, "|" .. tostring (s1) .. "|")
end

function navcomp:IsSectorProhibited (s1, useLogic)
	useLogic = useLogic or false
	local result = navcomp:IsEncounteredBotSector (s1) or navcomp:IsHostileBotSector (s1)
	if useLogic then
		result = result or (navcomp.data.avoidStormSectors and navcomp:IsEncounteredStormSector (s1))
		result = result or (navcomp.data.avoidManualSectors and navcomp:IsAvoidSector (s1))
	end
	
	return result
end

function navcomp:IsBlockableSector (s1, useLogic)
	useLogic = useLogic or false
	if useLogic and navcomp.data.avoidBlockableSectors then
		return navcomp:IsStationSector (s1) or navcomp:IsWormholeSector (s1)
	end
	
	return false
end

function navcomp:IsPathProhibited (s1, s2, useLogic)
	local path = navcomp:GetSectorLine (s1, s2)
	if #path == 2 then return false end
	local k, v
	for k,v in ipairs (path) do
		if v ~= s1 and v ~= s2 and navcomp:IsSectorProhibited (v, useLogic) then return true end
	end
	
	return false
end

function navcomp:IsCapitalSystem (sysId)
	if string.find (navcomp.data.capitalSystems, "|" .. tostring (sysId) .. "|") then
		return true
	else
		return false
	end
end

function navcomp:GetSectorStress (s1)
	local sysId, y, x = SplitSectorID (s1)
	local stress = 0
	local s2, i, j
	for i=x-1, x+1, 1 do
		for j=y-1, y+1, 1 do
			navcomp:Yield ()
			if i > 0 and j > 0 and i < 17 and j < 17 then
				s2 = navcomp:BuildSectorId (sysId, i, j)
				if navcomp:IsSectorProhibited (s2) or navcomp:IsAvoidSector (s2) then
					stress = stress + 1
				elseif navcomp:IsEncounteredStormSector (s2) then
					stress = stress + 1
				end
			elseif not navcomp:IsSectorProhibited (s1) then
				stress = stress - 1
			end
		end
	end

	if stress < 0 then stress = 0 end
	if navcomp:IsEncounteredStormSector (s1) then
		stress = stress + 1
	end
	if navcomp:IsSectorProhibited (s1) then
		stress = stress + 2
	end
	if navcomp:IsAvoidSector (s1) then
		stress = stress + 2
	end
	if navcomp:IsStarSector (s1) then
		stress = stress + 10
	end
	
	return stress
end

function navcomp:GetLineStress (s1, s2, map)
	local sectors = navcomp:GetSectorLine (s1, s2)
	local s
	local stress = 0
	for _, s in ipairs (sectors) do
		stress = stress + map [s]
	end
	
	return stress, #sectors
end

function navcomp:AnalyzeSystem (s1)
	local sysId = GetSystemID (s1)
	if not navcomp.data.stressMaps [sysId] then
		local s, o, i, j
		local stressMap = {}
		local systemStress = 0
		local step = 0
		for i=1, 16, 1 do
			for j=1, 16, 1 do
				step = step + 1
				s = navcomp:BuildSectorId (sysId, i, j)
				stressMap [s] = navcomp:GetSectorStress (s)
				systemStress = systemStress + stressMap [s]
				navcomp.ui.progress:Update (step)
				navcomp:Yield ()
			end
		end
		
		navcomp.data.stressMaps [sysId] =  {map = stressMap, stress = systemStress}
	end
	
	return navcomp.data.stressMaps [sysId]
end

function navcomp:Print (msg)
	local m = string.format ("\12700ff00%s\127o", msg)
	if navcomp.ui.progress then
		navcomp.ui.progress:SetTitle (m)
	end
	if PlayerInStation () or PDADialog.visible == "YES" then
		print (m)
	else
		HUD:PrintSecondaryMsg (m)
	end
end

function navcomp:PrintStats (sysId, stats)
	local s, s1, i, j
	for i=1, 16 do
		s = ""
		for j=1, 16 do
			s = s .. stats.map [navcomp:BuildSectorId (sysId, i, j)] .. "   "
		end
		print (s)
	end
end

function navcomp:Compare (n1, n2, s2, stats, area, proc)
	-- Check for Final Target
	local f, result
	if n1.s == s2 then
		return n1
	elseif n2.s == s2 then
		return n2
	end
	
	-- Check if path is clear to Target
	if not n1.blocked and (n2.blocked or n1.d2 < n2.d2) then
		return n1
	elseif not n2.blocked and (n1.blocked or n2.d2 < n1.d2) then
		return n2
	end

	-- Performing Waypoint Checks
	for _,f in ipairs (proc) do
		navcomp:Yield ()
		result = f (navcomp, n1, n2, s2, stats, area)
		if result then
			return result
		end
	end
	
	return n2
end

function navcomp:GetJumps (s1, s2, navrouteString, map)
	local sysId, x, y, x1, y1, ts, str1, str2, line1, line2
	local sectors = {}
	local areaStress = 0
	sysId, y1, x1 = SplitSectorID (s1)
	local areaSize = navcomp.plotter.metadata [sysId].areaSize
	for x=x1-navcomp:Lower (x1, areaSize), x1+navcomp:Upper (x1, areaSize) do
		for y=y1-navcomp:Lower (y1, areaSize), y1+navcomp:Upper (y1, areaSize) do
			ts = navcomp:BuildSectorId (sysId, x, y)
			if (ts == s2 or
						(not string.find (navrouteString, "|" .. ts .. "|") and 
						not navcomp:IsSectorProhibited (ts, true) and
						not navcomp:IsStarSector (ts) and
						not navcomp:IsConquerableStationSector (ts) and
						not navcomp:IsRacetrackSector (ts) and
						not navcomp:IsTrainingSector (ts) and
						not navcomp:IsBlockableSector (ts, true))) and
						not navcomp:IsPathProhibited (s1, ts, true) then
				str1, line1 = navcomp:GetLineStress (s1, ts, map)
				str2, line2 = navcomp:GetLineStress (ts, s2, map)
				areaStress = areaStress + map [ts]
				sectors [#sectors + 1] = {
					s = ts,
					d1 = navcomp:Distance (s1, ts),
					d2 = navcomp:Distance (ts, s2),
					str1 = str1,
					line1 = line1,
					str2 = str2,
					line2 = line2,
					blocked = navcomp:IsPathProhibited (ts, s2, true)
				}
			end
			navcomp:Yield ()
		end
	end
	
	return {sectors = sectors, stress = areaStress}
end