--[[
	Navigational Computer
	
	Author: Keller
]]
declare ("navcomp", {})
navcomp.version = "1.6.9 CUIC"
dofile ('cuic.lua')
dofile ("data/data.lua")
dofile ("data/data-star.lua")
dofile ("plotter/comparators.lua")
dofile ("data/data-plotter.lua")
dofile ("util.lua")
dofile ("ui/ui.lua")
dofile ("com/com.lua")
dofile ("debug.lua")

function navcomp:None () end

function navcomp:Help ()
	navcomp.pinfo ("NavComp Commands")
	navcomp.pinfo ("-  help - Prints this list")
	navcomp.pinfo ("-  options - Displays Control screen")
	navcomp.pinfo ("-  plot - Determines reasonable safe path between 2 sectors")
	navcomp.pinfo ("-  save - Saves current navpath")
	navcomp.pinfo ("-  load - Loads previously saved navpath")
	navcomp.pinfo ("-  clear - Clears previously recorded storm data or hostile bot data")
	navcomp.pinfo ("-  reset - Resets NavComp")
	navcomp.pinfo ("-  evade - Toggles Evasion")
	navcomp.pinfo ("-  autoreload - Activates autoreload for the current navpath")
end

function navcomp:Options ()
	navcomp.ui.control:CreateUI ()
end

-- Intended as a shortcut for external plugins to set paths without worrying about particulars
function navcomp:PlotPathToSector (sector)
	navcomp:ClearActivePath ()
	navcomp:PlotPath (sector)
end

function navcomp:PlotPath (sector)
	if sector then
		if type (sector) == "string" then
			sector = navcomp.data:GetSectorIdFromName (sector)
		end
		NavRoute.SetFinalDestination (sector)
	end
	navcomp.pda:SetEvasionMode (false)
	if navcomp.data.backgroundPlot then
		if not navcomp.data.plotter or coroutine.status (navcomp.data.plotter) == "dead" then
			-- Create plotter thread
			navcomp.data.plotter = coroutine.create (navcomp.DoPlot)
		end
		
		-- Start plotter thread
		if coroutine.status (navcomp.data.plotter) == "suspended" then
			navcomp:Print ("Plotting...")
			navcomp.ui.progress.visible = "YES"
			iup.Refresh (HUD.cboxlayer)
			navcomp:RunThread ()
		end
	else
		navcomp:DoPlot ()
		navcomp:Print ("Plot Complete")
	end
end

function navcomp:SavePath (args)
	navcomp.pda:SetEvasionMode (false)
	if args and args [2] then
		navcomp.data.activePath.name = args [2]
		navcomp.data.activePath.note = args [3] or ""
		navcomp.data.activePath.autoPlot = false
		if args [4] then
			navcomp.data.activePath.autoPlot = args [4]:lower () == "true"
		end
		navcomp.data.activePath.autoReload = false
		if args [5] then
			navcomp.data.activePath.autoReload = args [5]:lower () == "true"
		end
		navcomp.data.activePath.path = NavRoute.GetCurrentRoute ()
		navcomp:SaveActivePath (navcomp.data.activePath)
		navcomp:CheckAutoPlot (navcomp.data.activePath)
	else
		local path1 = navcomp.data:Clone (navcomp.data.activePath.path)
		local path2 = GetFullPath (GetCurrentSectorid (), NavRoute.GetCurrentRoute ())
		if path2 and #path2 > 0 then
			if path1 [1] == GetCurrentSectorid () then table.remove (path1, 1) end
			table.remove (path2, 1)
		
			if not navcomp:ComparePaths (path1, path2) then
				navcomp:ClearActivePath ()
			end
			if not navcomp.data.activePath.name then
				local first = GetCurrentSectorid ()
				if first then first = AbbrLocationStr (first)
				else first = "No Sector" end
				local last = NavRoute.GetFinalDestination ()
				if last then last = AbbrLocationStr (last)
				else last = "No Sector" end
				navcomp.data.activePath.name = first .. ":" .. last
			end
			navcomp.ui:CreateUI (false)
		else
			navcomp.ui:CreateAlertUI ("No Path to Save")
		end
	end
end

function navcomp:LoadPath (args)
	if args and args [2] then
		navcomp:LoadActivePath (args [2])
	else
		navcomp.pda:SetEvasionMode (false)
		navcomp.ui:CreateUI (true)
	end
end

function navcomp:Exchange ()
	navcomp.com.ui:CreateUI ()
end

function navcomp:ClearStorms ()
	navcomp.ui:CreateClearDataUI ()
end

function navcomp:ToggleEvasionMode ()
	if not PlayerInStation () then
		navcomp.data.isEvading = not (navcomp.data.isEvading and true)
		if navcomp.data.isEvading then
			-- Determine where we are in the existing path (if any)
			navcomp.data.activePath.path = NavRoute.GetCurrentRoute ()
			
			-- Start new evasion path
			NavRoute.clear ()
			navcomp:PlotEvasionPath ()
		else
			-- Reset path back to its original course
			navcomp:SetPath (navcomp.data.activePath.path)
		end
		navcomp.pda:SetEvasionMode (navcomp.data.isEvading)
	end
end

function navcomp:Reset ()
	-- Disables the current path if it's set to autoreload
	if navcomp.data.activePath.autoReload then
		navcomp:ClearActivePath ()
	end
end

function navcomp:AutoReloadCurrentPath ()
	navcomp.data.activePath.path = GetFullPath (GetCurrentSectorid (), NavRoute.GetCurrentRoute ())
	navcomp.data.activePath.autoReload = true
end

navcomp.arguments = {
	analyze = navcomp.Analyze,
	find = navcomp.FindPath,
	line = navcomp.Line,
	jumps = navcomp.Jumps,
	metadata = navcomp.MetaData,
	segment = navcomp.Segment,
	help = navcomp.Help,
	options = navcomp.Options,
	plot = navcomp.PlotPath,
	load = navcomp.LoadPath,
	save = navcomp.SavePath,
	reset = navcomp.Reset,
	clear = navcomp.ClearStorms,
	exchange = navcomp.Exchange,
	evade = navcomp.ToggleEvasionMode,
	autoreload = navcomp.AutoReloadCurrentPath
}
function navcomp.Start (obj, args)
	if args then
		local f = navcomp.arguments [args [1]:lower ()] or navcomp.Help
		f (navcomp, args)
	else
		navcomp:Help ()
	end
end
RegisterUserCommand ("navcomp", navcomp.Start)
RegisterUserCommand ("nc", navcomp.Start)
RegisterUserCommand ("evade", navcomp.ToggleEvasionMode)

return navcomp