--[[
	Rook Move
	
	A specialized comparator looking for paths along the outside edges of a system
	
	There are 3 scenarios:
		2 Jump (2 paths - at the row/col intersections between the endpoints)
		3 Jump (4 paths - out to an edge, along that edge, then in to the 2nd endpoint)
		4 Jump (6 paths - out to an edge, along that edge, along another edge, then in to the 2nd endpoint)
]]

local function Check2Jump (n1, n2, s2)
	-- Check for row/col jump point for 2 jump scenario
	local sysId
	local w1 = {x=0, y=0, c1={s=0, d=0}, c2={s=0, d=0}, ok1=true, ok2=true}
	local w2 = {x=0, y=0, c1={s=0, d=0}, c2={s=0, d=0}, ok1=true, ok2=true}
	local w3 = {x=0, y=0, c1={s=0, d=0}, c2={s=0, d=0}, ok1=true, ok2=true}
	local w4 = {x=0, y=0, c1={s=0, d=0}, c2={s=0, d=0}, ok1=true, ok2=true}
	local s = {x=0, y=0}
	
	-- Get coordinates of the considered waypoint and the target
	navcomp:Yield ()
	sysId, w1.y, w1.x = SplitSectorID (n1.s)
	sysId, w2.y, w2.x = SplitSectorID (n2.s)
	sysId, s.y, s.x = SplitSectorID (s2)
	
	-- 2 Jump Scenarios
	-- Find the common point c1.
	-- Check if the path is clear from n1 to c1 and c1 to s2
	w1.c1.s = navcomp:BuildSectorId (sysId, w1.x, s.y)
	w1.c1.d = (w1.x - s.x)*(w1.x - s.x)
	w1.c2.s = navcomp:BuildSectorId (sysId, s.x, w1.y)
	w1.c2.d = (w1.y - s.y)*(w1.y - s.y)
	w2.c1.s = navcomp:BuildSectorId (sysId, w2.x, s.y)
	w2.c1.d = (w2.x - s.x)*(w2.x - s.x)
	w2.c2.s = navcomp:BuildSectorId (sysId, s.x, w2.y)
	w2.c2.d = (w2.y - s.y)*(w2.y - s.y)
	
	navcomp:Yield ()
	w1.ok1 = not (navcomp:IsPathProhibited (n1.s, w1.c1.s, true) or navcomp:IsPathProhibited (w1.c1.s, s2, true))
	w1.ok2 = not (navcomp:IsPathProhibited (n1.s, w1.c2.s, true) or navcomp:IsPathProhibited (w1.c2.s, s2, true))
	local ok1 = w1.ok1 or w1.ok2
	w2.ok1 = not (navcomp:IsPathProhibited (n1.s, w2.c1.s, true) or navcomp:IsPathProhibited (w2.c1.s, s2, true))
	w2.ok2 = not (navcomp:IsPathProhibited (n1.s, w2.c2.s, true) or navcomp:IsPathProhibited (w2.c2.s, s2, true))
	local ok2 = w2.ok1 or w2.ok2
	--print (string.format ("W1: C1 s= %s, d=%d\tC2 s=%s, d=%d\t%s", AbbrLocationStr (w1.c1.s), w1.c1.d, AbbrLocationStr (w1.c2.s), w1.c2.d, tostring (ok1)))
	--print (string.format ("W2: C1 s= %s, d=%d\tC2 s=%s, d=%d\t%s", AbbrLocationStr (w2.c1.s), w2.c1.d, AbbrLocationStr (w2.c2.s), w2.c2.d, tostring (ok2)))
	
	-- If valid, return n1
	if ok1 and (not ok2 or 
				w1.c1.d < w2.c1.d or
				w1.c1.d < w2.c2.d or
				w1.c2.d < w2.c1.d or
				w1.c2.d < w2.c2.d) then
		return n1
	end
end

local function GetNearsFars (n1)
	local sysId, x, y, nx, ny, fx, fy
	sysId, y, x = SplitSectorID (n1.s)
	nx = 1
	fx = 16
	if x > 8 then
		nx = 16
		fx = 1
	end
	ny = 1
	fy = 16
	if y > 8 then
		ny = 16
		fy = 1
	end
	
	return x, y, nx, ny, fx, fy
end

local function CheckForOpenEdge (n1)
end

local function Calc3Jump (n1, s2)
	-- Find the near edges
	-- nearX and nearY are closest X and Y to starting point
	-- farX and farY are closest X and Y to end point
	--
	-- Routes become:
	--	1) n1 -> (x, ny) -> (sx, ny) -> s2
	--	2) n1 -> (fx, y) -> (fx, sy) -> s2
	--	3) n1 -> (nx, y) -> (nx, sy) -> s2
	--	4) n1 -> (x, fy) -> (sx, fy) -> s2
	-- Algorithm will return the first clear path of set
	local sysId, sy, sx = SplitSectorID (s2)
	
	-------------------------------------------------------------------------------------------------
	-- Check node jump locations
	-------------------------------------------------------------------------------------------------
	local x, y, nx, ny, fx, fy = GetNearsFars (n1)
	
	-- Check each possible path for acceptance of current waypoint choice
	local result = not (navcomp:IsPathProhibited (n1, navcomp:BuildSectorId (sysId, x, ny), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, x, ny), navcomp:BuildSectorId (sysId, sx, ny), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, sx, ny), s2, true))
						
	result = result or not (navcomp:IsPathProhibited (n1, navcomp:BuildSectorId (sysId, fx, y), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, fx, y), navcomp:BuildSectorId (sysId, fx, sy), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, fx, sy), s2, true))
						
	result = result or not (navcomp:IsPathProhibited (n1, navcomp:BuildSectorId (sysId, nx, y), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, nx, y), navcomp:BuildSectorId (sysId, nx, sy), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, nx, sy), s2, true))
						
	result = result or not (navcomp:IsPathProhibited (n1, navcomp:BuildSectorId (sysId, x, fy), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, x, fy), navcomp:BuildSectorId (sysId, sx, fy), true) or
						navcomp:IsPathProhibited (navcomp:BuildSectorId (sysId, sx, fy), s2, true))
						
	return result
end

local function Check3Jump (n1, n2, s2)
	-- 3 Jump Scenarios
	-- Check n1
	local result1 = Calc3Jump (n1, s2)
	-- Check n2
	local result2 = Calc3Jump (n2, s2)
						
	-- If n1 has solution and n2 does not -> n1
	-- All other cases -> nil (since n2 is the default node utilized)
	if result1 and not result2 then
		return n1
	end
end

local function Calc4Jump (n1, s2)
	-- There are 4 paths for each considered node
	-- 6 paths that need checking
	-- n1 -> (x, ny) -> (fx, ny) -> (fx, sy) -> s2  (yxyx)
	-- n1 -> (x, ny) -> (nx, ny) -> (nx, sy) -> s2  (yxyx)
	-- n1 -> (nx, y) -> (nx, fy) -> (sx, fy) -> s2  (xyxy)
	-- n1 -> (nx, y) -> (nx, ny) -> (sx, ny) -> s2  (xyxy)
	-- n1 -> (fx, y) -> (fx, fy) -> (sx, fy) -> s2  (xyxy)
	-- n1 -> (x, fy) -> (fx, fy) -> (fx, sy) -> s2  (yxyx)
	-- Any path which contained crossing jumps was ruled out since it could be
	-- managed by a path with fewer jumps
	local sysId, sy, sx = SplitSectorID (s2)
	
	-------------------------------------------------------------------------------------------------
	-- Check node jump locations
	-------------------------------------------------------------------------------------------------
	local x, y, nx, ny, fx, fy = GetNearsFars (n1)
	
end

local function Check4Jump (n1, n2, s2)
	-- 4 Jump Scenarios
end

function navcomp.plotter:RookMove (n1, n2, s2, stats, area)
	local result = Check2Jump (n1, n2, s2)
	if result then return result end
	
	result = Check3Jump (n1, n2, s2)
	if result then return result end
	
	result = Check4Jump (n1, n2, s2)
	if result then return result end
end